#!/usr/bin/perl -w    

                 
use Carp;
use Getopt::Std;
use File::Basename;
$cpus = 12;

my $MAN = <<BLAH;

The script performs cross-validation analysis for
nucleotide sequences, in order to determine the best
fitting model.

synopsis: cross.pl argument1
argument1: name of the input alignment

please edit the variable "cpus" at the top of the script
to specify how many CPU cores of a server you wish to load.

phylobayes executable - named "pb",
cvrep executable - named "cvrep",
readcv executable - named "readcv",
readpb executable - named "readpb"
sumcv executable - named "sumcv"
should be in your path


BLAH

$#ARGV==0 || croak ($MAN);
$infile = $ARGV[0];



push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -cat -dgam 4 -covext   $infile.catdgam4covext";
push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -cat -dgam 4 -covarion $infile.catdgam4covarion";
push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -cat -dgam 4	      $infile.catdgam4";
push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -ncat 1 -dgam 4 -covext   $infile.ncat1dgam4covext";
push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -ncat 1 -dgam 4 -covarion $infile.ncat1dgam4covarion";
push @do_pb, "pb -f  -d $infile -x 1 1000 -gtr -ncat 1 -dgam 4  	 $infile.ncat1dgam4";


push @do_readpb, "readpb -x 300 1 1000 $infile.catdgam4covarion";
push @do_readpb, "readpb -x 300 1 1000 $infile.catdgam4covext";
push @do_readpb, "readpb -x 300 1 1000 $infile.catdgam4";
push @do_readpb, "readpb -x 300 1 1000 $infile.ncat1dgam4covarion";
push @do_readpb, "readpb -x 300 1 1000 $infile.ncat1dgam4covext";
push @do_readpb, "readpb -x 300 1 1000 $infile.ncat1dgam4";


for $i (0.. $#do_pb)
{
  print "$do_pb[$i]\n";
}


for $i (0.. $#do_readpb)
{
  print "$do_readpb[$i]\n";
}

thread(@do_pb);
thread(@do_readpb);


system "cvrep -nrep 10 -nfold 10 -d $infile replica";
print "cvrep -nrep 10 -nfold 10 -d $infile replica\n";
system "ls *learn.ali > qwertzuiop";
-e "qwertzuiop" || croak (sprintf ("cannot find qwertzuiop file"));
open (LIST, "<qwertzuiop") or croak (sprintf ("cannot open qwertzuiop file for reading"));
while (defined($stroka = <LIST>))
{
   print $stroka;
   chop $stroka;
   push @replicas, $stroka;
}


$for_sumcv = "";

for $i (0.. $#do_pb)
{
   
   $do_pb[$i] =~ m/1000(.+)$infile\.(.+)/;
   print "AAA $1 $2\n";

   $tree = "$infile.$2"."_sample.con.tre";
   $prefix = $1;
   $prefix1 = $2;
   print "PREFIX $prefix\n";
   print "PREFIX1 $prefix1\n";
   $for_sumcv = $for_sumcv." $prefix1 ";
   
   $count = -1;
   for $i (0.. $#replicas)
   {
      $count++;
      $command = "pb -d $replicas[$i] -T $tree -x 1 200 $prefix $prefix1$replicas[$i]";
      print "$command\n";
      push @commands, $command; 
      
      
      $command2 = "readcv -rep $count -x 50 1 $prefix1 replica";
      print "$command2\n";
      push @commands2, $command2;       
      
        
   }
}
print "SEE $for_sumcv\n";
thread(@commands);
print "###############\n";
thread(@commands2);
system "sumcv -nrep 10 $for_sumcv replica > xxCVOUTPUT";
sub thread 
{
   $num2 = $#_ + 1;
   $number = 0;
   if ($#_ + 1 < $cpus)
   {
      $num1 = $#_ + 1;
   }
   else
   {
      $num1 = $cpus;
   } 
   for $i (1.. $num1)
   {
      $pid = fork();
      if ($pid) 
      {
         # parent
         $number++;
         print "RUNNING CHILD PROCESS NUMBER: $number\n";
         $children{$number} = $pid;
      } 
      elsif ($pid == 0) 
      {
         # child
         system "$_[$i-1]";
         exit(0);
      } 
      else 
      {
         die "couldnt fork: $!\n";
      }
   }
   if ($num2 > $cpus)
   {
      for $i ($num1+1.. $num2)
      {
         for $pid_nr (sort {$a <=> $b} keys %children) 
         {
            print "PID_N $children{$pid_nr}\n";
            waitpid($children{$pid_nr}, 0);
            delete $children{$pid_nr};

            $pid = fork();
            if ($pid) 
            {
               # parent
               $number++;
               print "STARTING NEW CHILD $number\n";
               $children{$number} = $pid;
            } 
            elsif ($pid == 0) 
            {
               # child
               system "$_[$i-1]";
               exit(0);
            }
            else 
            {
               die "couldnt fork: $!\n";
            } 
            last;
         }
      }
   }
   for $pid_nr (sort {$a <=> $b} keys %children) 
   { 
      waitpid($children{$pid_nr}, 0);
   }
}

system "rm qwertzuiop";
