#!/usr/bin/perl -w    

                 
use Carp;
use Getopt::Std;
use File::Basename;
$dist_measure = "p";


$full_matrix = 0;
#$full matrix variable determines how the OV measure is calculated:
#
#if set to 0 (default), OV at a given alignment position is calculated as
#sum of differences between all different individual OV values observed in pair-wise
#comparisons in the square matrix for this position, divided by number of pair-wise comparisons between
#informative characters in this position (excluding gaps and N bases).
#
#if set to 1, OV will be calculated as above, but division will be made by the number of all members
#of the square matrix, including all non-informative character states.
#
#If input alignment has plenty of gaps and undefined characters, second option (1) might be considered.
#
#OV algorithm most efficiently finds and concentrates saturated position towards the divergent sorted alignment end
#when there are as few as possible gaps and undefined character states in the data.
#
#efficiency of sorting with both options can be judged by looking at the *deviation output files 
#which are produced by noisereductor.pl, which contain deviations of the mean p-distances from mean
#evolutionary distances calculated based on divergent alignment partitions.




my $MAN = <<BLAH;

This script performs observed variability-based character sorting as described in: 
Goremykin VV, Nikiforova SV, Bininda-Emonds ORP, Automated removal of noisy
data in phylogenomic analyses. JME 2010. 71:319-331.

In addition it can sorts alignment positions according to 
the proxy of the substitution rate supplied in the file
named "sortme_so", which should be in the current directory.
*A.plot and *L.plot output files of the program COMPASS can be directly
re-named to "sortme_so" and used to sort positions in the alignment
according to the chosen measure of variability.

If the file "sortme_so" is not present in the current directory, the positions 
will be sorted by the observed variability at every alignment position

After sorting, the script creates series of A, B and C partitions which 
can be used in the correlation analyses.

synopsis: sorter.pl arg1 arg2 arg3 arg4
arg1:	name of input alignment in nexus, CLUSTAL or fasta formats (produced by SEAVIEW)
arg2:	integer - size [pos.] of each shortening step. 
arg3:	"nexus"/"phylip"/"fasta" - format of the A,B and C partitions to be created. 
arg4:	base file name of the A,B and C partitions to be created.

Note:

"phylip" format of the partitions is suitable for PHYML program. 
This format might not be suitable for other programs.

BLAH

$#ARGV>=3 || die ($MAN);
$infile = $ARGV[0];
$step = $ARGV[1];
$outformat = $ARGV[2];
$outfile = $ARGV[3];


open (VVOD, "<$infile") or croak (sprintf ("cannot open $infile for reading"));
open DDD, ">ddd";
#################################################
#reading the alignment in nexus, clustal or fasta formats 
#(all produced by SEAVIEW)
#################################################
while (defined($stroka = <VVOD>))
{
   $seq = "";
   if ($stroka =~ /^#NEXUS/)
   {
      print "format is nexus\n";
      $stroki = 0;
      while (defined($stroka = <VVOD>))
      {
         $stroki++;
         if ($stroki == 1)
         { 
            if ($stroka =~ /saved by seaview/)
            {
               print "NEXUS seaview format\n";
               while (defined($stroka = <VVOD>))
               {
        	  mark11: if ($stroka =~ /^\[\d+\] \'(.+)\'/ or $stroka =~ /^\[\d+\] (.+)/)
        	  {
		
		     print $stroka;
		     $fastacomment = $1;
        	     $fastacomment =~ s/\r//g;
		     $fastacomment =~ s/\n//g;
		     
        	     $count++;
        	     $xashnames{$count} = $fastacomment;
        	     $seq = "";
        	     while (defined($stroka = <VVOD>) and $stroka !~ /^\[|^\;/)
        	     {
        		$stroka =~ s/\r//g;
			$stroka =~ s/\n//g;
        		$stroka =~ s/ +//g;
        		$seq = $seq.$stroka;
        	     }
		     $alnlength = length $seq;
        	     @stroka = split (//, $seq);
        	     $xash{$count} = [ @stroka ];
        	     if (defined $stroka){$stroka =~ /^\[/ and goto mark11;}
        	  }
               }
            }	
	    else
	    {
	       print "NEXUS sequential format\n";
	       $allowed = 0;
	       while (defined($stroka = <VVOD>))
	       {
		 $stroka =~ /matrix/ and $allowed = 1;
		 if ($allowed == 1 and $stroka =~ /^([^ ]+)\s+([^ ]+)/)
		 {
		   $count++;
		   $xashnames{$count} = $1;
		   $seq = $2;
        	   $seq =~ s/\r//g;
		   $seq =~ s/\n//g;
		   $alnlength = length $seq;
		   @stroka = split (//, $seq);
		   $xash{$count} = [ @stroka ];
		 }
	       }
	    }
         }
      }
   }
   elsif ($stroka =~ /^CLUSTAL/)
   {
      while (defined($stroka = <VVOD>))
      {       
     	  if ($stroka =~ /^([^ ]+) +([\w-]{1,60})/)
     	  {
     	      $empti = 0;
     	      $count++;
	      $xashnames{$count} = $1;
     	      $line = $2;
              $line =~ s/\r//g;
              $line =~ s/\n//g;
	      @line = split (//, $line);
     	      push (@{$xash{$count}}, @line);
     	  }
     	  else
     	  {
     	     $empti++;
     	     $empti == 1 and $lastcount = $count;
     	     $count = 0;
     	  }
      }
      $alnlength = $#{$xash{1}} + 1;
      $count = $lastcount;    
   }
   else
   {
      mark1: if ($stroka =~ /^\>(.+)/)
      {
         
	 $fastacomment = $1;
         $fastacomment =~ s/\r//g;
         $fastacomment =~ s/\n//g;
	 print DDD "$fastacomment\t";
	 $count++;
         $xashnames{$count} = $fastacomment;
	 $seq = "";
         while (defined($stroka = <VVOD>) and $stroka !~ /^>/)
         {
            
            $stroka =~ s/\r//g;
	    $stroka =~ s/\n//g;
	    $stroka =~ s/ *$//;
	    $seq = $seq.$stroka;
         }
	 print DDD "$seq\n";
         $alnlength = length $seq;
         @stroka = split (//, $seq);
         $xash{$count} = [ @stroka ];
         if (defined $stroka){$stroka =~ /^\>(.+)/ and goto mark1;}
         
      }
   }
}
close VVOD;
#die;
#################################################
#attempting to read a *.plot output file of COMPASS (re)named "sortme_so";
#################################################


$switch = 0;
$control = open (VVOD, "<sortme_so") or goto mark20;
while (defined($stroka = <VVOD>))
{
   $seq = "";
   if ($stroka =~ /^(.+)\n/)
   {
      if ($1 > 1)
      {
	 $switch = 1;       
      }
      $countchars++;
      $dur = $countchars - 1;
      print "pos. $countchars\tproxy of subst. rate is $1\n";
      for $species (sort {$a <=> $b} keys %xash)
      {	   
   	   push @{$xash_out{$species}{$1}}, $xash{$species}[$dur];
      }
   }
}
close VVOD;

#################################################
#computing distance at each position if "sortme_so" cannot be read
#################################################
mark20:

unless (defined $control)
{
for $d (0.. $alnlength - 1)
{
   $distance = 0; $matrix_members = 0;$matrix_members1 = 0;
   for $species (keys %xash)
   {   
      for $i (1.. $count)
      {
	 if ($species != $i)
	 {
	    $matrix_members++;
	    if ($xash{$species}[$d] ne "-" and $xash{$i}[$d] ne "-"
	    and $xash{$species}[$d] ne "N" and $xash{$i}[$d] ne "N" and
	    $xash{$species}[$d] ne "n" and $xash{$i}[$d] ne "n")
	    {
	       $matrix_members1++;
	       if ($xash{$species}[$d] ne $xash{$i}[$d]) 
	       {
	          $distance++;		   
	       }
	    }   
	 }
      }
   }
   if ($matrix_members1 == 0)
   {
      $total_dist = 0;
   }
   elsif ($dist_measure eq "jc")
   {
      $full_matrix == 1 and $p = $distance/$matrix_members;
      $full_matrix == 0 and $p = $distance/$matrix_members1;
      print "$p\n";
      $p >= 0.75 and $p = 0.749999999;
      $total_dist = -3/4*log(1 - 4/3*($p));
   }
   elsif ($dist_measure eq "p")
   {
      $full_matrix == 1 and $total_dist = $distance/$matrix_members;
      $full_matrix == 0 and $total_dist = $distance/$matrix_members1;
              
   }   
   $posnum = $d+1;
   print "pos. $posnum\tOV is $total_dist\n";
   for $species (sort {$a <=> $b} keys %xash)
   {	  
       push @{$xash_out{$species}{$total_dist}}, $xash{$species}[$d];
   }
}   
}





for $species (sort {$a <=> $b} keys %xash_out)
{ 
   $zamena++;
   for $total_dist (sort {$a <=> $b} keys %{$xash_out{$species}})
   {
      for $l (0.. $#{$xash_out{$species}{$total_dist}})
      {
         push @{$xash_out1{$species}}, $xash_out{$species}{$total_dist}[$l];
	 push @{$xash_dist{$zamena}}, $total_dist;
      }
   }
}
$step1 = 0;
while (defined %xash_out1)
{
   $alnlength1 =  $alnlength - $step1;
   $switch == 0 and $distance_cut = sprintf ("%.4f", $xash_dist{1}[$alnlength1 - 1]);
   $switch == 1 and $distance_cut = $xash_dist{1}[$alnlength1 - 1];
   
   #print "$xash_dist{1}[$alnlength1 - 1]\t ### $distance_cut\n";
   open (VIVODB , ">$outfile.$alnlength1.$distance_cut"); 
   if ($outformat eq "fasta")
   {
      for $species (sort {$a <=> $b} keys %xash_out1)
      {
         print VIVODB ">$xashnames{$species}\n";
         for $l (0.. $#{$xash_out1{$species}})
         {      
            print VIVODB "$xash_out1{$species}[$l]";
         }
         print VIVODB "\n";
      }   
      close VIVODB;
   }
   elsif ($outformat eq "phylip")
   {
      print VIVODB "$count   $alnlength1\n";
      for $species (sort {$a <=> $b} keys %xash_out1)
      {	 
	 $dlina = length $xashnames{$species};
	 $ostatok = 30 - $dlina; 
	 
	 print VIVODB "$xashnames{$species}";
	 for $b (1.. $ostatok)
	 {
	    print VIVODB " "; 
	 }
	 for $l (0.. $#{$xash_out1{$species}})
         {      
            print VIVODB "$xash_out1{$species}[$l]";	    
         }
         print VIVODB "\n";
      }
      close VIVODB;   
   }
   elsif ($outformat eq "nexus")
   {
      print VIVODB "#NEXUS\n\n";
      print VIVODB "begin data\;\n";
      print VIVODB "   dimensions ntax=$count nchar=$alnlength1\;\n";
      print VIVODB "   format datatype=Nucleotide interleave=no gap=-\;\n";
      print VIVODB "   matrix\n";
      for $species (sort {$a <=> $b} keys %xash_out1)
      {	 
	 $dlina = length $xashnames{$species};
	 $ostatok = 30 - $dlina; 
	 
	 print VIVODB "$xashnames{$species}";
	 for $b (1.. $ostatok)
	 {
	    print VIVODB " "; 
	 }
	 for $l (0.. $#{$xash_out1{$species}})
         {      
            print VIVODB "$xash_out1{$species}[$l]";
         }
         print VIVODB "\n";
      }
      print VIVODB "\;\n";
      print VIVODB "end\;\n";
      close VIVODB;   
   }  
   $hhh = 0;$udaleno = 0;
   for $species (sort {$a <=> $b} keys %xash_out1)
   {
      $hhh++;
      if ($#{$xash_out1{$species}} >= $step)
      {
         for (1.. $step)
         {
	    $udaleno++;
	    $deleted_character = pop @{$xash_out1{$species}};
	    push @{$rest{$species}}, $deleted_character;
         }
      }
      else
      {
         undef %xash_out1;
      }	 
   }   
   $udaleno1 = $udaleno/$hhh;
   $step1 = $step1 + $udaleno1;

   $switch == 0 and $distance_cut1 = sprintf ("%.4f", $xash_dist{1}[$alnlength1 - 1 - $udaleno1]);
   $switch == 1 and $distance_cut1 = $xash_dist{1}[$alnlength1 - 1 - $udaleno1];



   $blah = $alnlength1-$udaleno1;
   
   
   
   open (VIVODB, ">$outfile.$blah.deleted.$step1.$distance_cut1");
   open (VIVODN, ">$outfile.$blah.justdeleted.$step1.$distance_cut1");   
   if ($outformat eq "fasta")
   {
      for $species (sort {$a <=> $b} keys %rest)
      {  
         $printed = 1;
         print VIVODB ">$xashnames{$species}\n";
         for ($l = $#{$rest{$species}}; $l >= 0; $l--)
         {      
            
	    print VIVODB "$rest{$species}[$l]";
	    if ($printed <= $step)
	    {
	       print VIVODN "$rest{$species}[$l]";
	       $printed++
	    }
         }
         print VIVODB "\n";print VIVODN "\n";
      }   
      close VIVODB;close VIVODN;
   }
   elsif ($outformat eq "phylip")
   {
      print VIVODB "$count   $step1\n";
      print VIVODN "$count   $step\n";
      
      for $species (sort {$a <=> $b} keys %rest)
      {	 
	 $dlina = length $xashnames{$species};
	 $ostatok = 30 - $dlina; 
         $printed = 1;
	 
	 print VIVODB "$xashnames{$species}";print VIVODN "$xashnames{$species}";
	 for $b (1.. $ostatok)
	 {
	    print VIVODB " ";print VIVODN " "; 
	 }
	 for ($l = $#{$rest{$species}}; $l >= 0; $l--)
         {      
            print VIVODB "$rest{$species}[$l]";
	    if ($printed <= $step)
	    {
	       print VIVODN "$rest{$species}[$l]";
	       $printed++
	    }
         }
         print VIVODB "\n";print VIVODN "\n";
      }
      close VIVODB; close VIVODN;  
   }
   elsif ($outformat eq "nexus")
   {
      print VIVODB "#NEXUS\n\n";
      print VIVODB "begin data\;\n";
      print VIVODB "   dimensions ntax=$count nchar=$step1\;\n";
      print VIVODB "   format datatype=Nucleotide interleave=no gap=-\;\n";
      print VIVODB "   matrix\n";
      
      
      print VIVODN "#NEXUS\n\n";
      print VIVODN "begin data\;\n";
      print VIVODN "   dimensions ntax=$count nchar=$step\;\n";
      print VIVODN "   format datatype=Nucleotide interleave=no gap=-\;\n";
      print VIVODN "   matrix\n";
      
      
      for $species (sort {$a <=> $b} keys %rest)
      {	 
	 $dlina = length $xashnames{$species};
	 $ostatok = 30 - $dlina; 
	 $printed = 1;

	 print VIVODB "$xashnames{$species}";print VIVODN "$xashnames{$species}";
	 for $b (1.. $ostatok)
	 {
	    print VIVODB " ";print VIVODN " "; 
	 }
	 for ($l = $#{$rest{$species}}; $l >= 0; $l--)
         {      
            print VIVODB "$rest{$species}[$l]";
	    if ($printed <= $step)
	    {
	       print VIVODN "$rest{$species}[$l]";
	       $printed++
	    }
         }
         print VIVODB "\n";print VIVODN "\n";
      }
      print VIVODB "\;\n";
      print VIVODB "end\;\n";
      close VIVODB;
      
      print VIVODN "\;\n";
      print VIVODN "end\;\n";
      close VIVODN;      
         
   }    
}
system "rm *0.0000  *.0";





















